<?php

namespace MGModule\ipmanager2\app\modules;
use MGModule\ipmanager2\app\core;

/* * ********************************************************************
 *
 *
 *  CREATED BY MODULESGARDEN       ->        http://modulesgarden.com
 *  AUTHOR                         ->       dominik@modulesgarden.com
 *  CONTACT                        ->       contact@modulesgarden.com
 *
 *
 *
 * This software is furnished under a license and may be used and copied
 * only  in  accordance  with  the  terms  of such  license and with the
 * inclusion of the above copyright notice.  This software  or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person.  No title to and  ownership of the  software is  hereby
 * transferred.
 *
 *
 * *********************************************************************/

/**
 * AutoRelease 3rd Party Submodule
 *
 * @author Dominik Gacek <dominik@modulesgarden.com>
 * @link http://modulesgarden.com ModulesGarden - Top Quality Custom Software Development
 * @license http://www.modulesgarden.com/terms_of_service
 */

class AutoreleaseIntegration extends core\IntegrationModule
{
    /**
     * Integration module name
     *
     * @return string
     */
    public function getName()
    {
        return 'Test Integration Module';
    }

    /**
     * Integration module description
     *
     * @return string
     */
    public function getDescription()
    {
        return 'A test integration module that logs information about the IP Addresses processing.';
    }

    /**
     * Integration module type
     * @important based on this type "provision" servers will be identified so use here
     * produc module name (lowercase !)
     *
     * @return string
     */
    public function getModuleType()
    {
        return 'autorelease';
    }

    /**
     * if true, this function will be executed:
     *     $integration->modulePreCreate($serviceid, $data->params['moduletype'], $data);
     *     before on WHMCS hook `PreModuleCreate`
     *
     * @var boolean
     */
    public $preCreateAction = false;


    /**
     * Once this is changed to true
     * Every Assign of ip address will be trigerred
     * and as a result method `setIPAddress` on integration module will be trigerred as well on hook `AfterModuleCreate`
     *
     * @var boolean
     */
    public $forceAssignAfterCreate = false;


    /**
     * Function run on each IP address that will be set
     * executed on hook 'AfterModuleCreate'
     *
     * @param string    $ip              generated ip address that will be assigned
     * @param object    $service         Hosting object to what this ip address will be assigned to
     * @param object    $server          Server related to this hosting (if is set)
     * @param object    $pool            IP Manager poll object, from what this ip address was taken
     * @param integer   $iterationNumber lets say we have 10 ip address to assign, this is done in loop, where each iteration number is passed by this variable
     * @return boolean|string
     */
    public function setIPAddress($ip, $service, $server, $pool, $iterationNumber = 1)
    {
        // just add log
        core\Logger::factory()->log(sprintf('New IP address assigned: %s, Service id: %s, Server id: %s, It was a %d iteration of the integration', $ip, $service->id, $server->id, $iterationNumber));

        return true;
    }

    /**
     * Function run on each IP address that will be set
     *
     * @param string    $ip              generated ip address that will be unassigned from service
     * @param object    $service         Hosting object to what this ip address was removed from
     * @param object    $pool            IP Manager poll object, from what this ip address was taken
     * @return boolean|string
     */
    public function unsetIPAddress($ip, $service, $server)
    {
        // just add log
        core\Logger::factory()->log(sprintf('IP address unassigned: %s, Service id: %s, Server id: %s', $ip, $service->id, $server->id));

        return true;
    }

    /**
     * Similar to setIPAddress, but
     * executed on hook 'PreModuleCreate'
     *
     * @param string    $ip              generated ip address that will be assigned
     * @param object    $service         Hosting object to what this ip address will be assigned to
     * @param object    $server          Server related to this hosting (if is set)
     * @param object    $pool            IP Manager poll object, from what this ip address was ta
     * @param integer   $iterationNumber lets say we have 10 ip address to assign, this is done in loop, where each iteration number is passed by this variable
     * @return boolean|string
     */
    public function moduleCreatePreIPSet($ip, $service, $server, $pool, $iterationNumber)
    {
        // you can do something here
        return false;
    }
}
